import json
import logging
import traceback
from dataclasses import asdict, dataclass
from typing import Any, Optional

import requests_oauthlib as oauth

log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)


@dataclass
class NetsuiteObject:
    url: Optional[str] = None
    request_headers: Optional[dict] = None
    request_data: Optional[dict | str] = None
    response: str = None
    code: int = None

    def __repr__(self):
        return f"NetsuiteObject(url={self.url}, request_headers={self.request_headers}, request_data={self.request_data}, response={self.response}, code={self.code})"

    @property
    def json(self):
        return asdict(self)


class NetSuite:
    """
    The NetSuite class is a Python wrapper for the NetSuite REST API. It provides methods for making HTTP requests to the NetSuite REST API using OAuth 1.0 authentication. The class supports GET, PUT, POST, and DELETE HTTP methods. The class also provides error handling for failed requests.
    ```
    from NetSuite_Connector.NetSuite import NetSuite
    nt = NetSuite(
        account_id=123456,
        consumer_keys=dict(consumer_key="2345678", consumer_secret="3456yhg"),
        token_keys=dict(token_key="wfdbfdsdfg", token_secret="efguhfjoidejhfije"),
    )

    x = nt.get(
        url="https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx",
        headers={"Content-Type": "application/json"},
        params={}
    )
    print(x)
    # NetsuiteObject(url='https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx', request_headers={'Content-Type': 'application/json'}, response='{"foo":"bar"}', code=200)
    ```
    """

    def __init__(self, account_id: Any, consumer_keys: dict, token_keys: dict) -> None:
        self.oauth_version = "1.0"
        self.signature_method = "HMAC-SHA256"
        self.account_id = account_id
        self.consumer_key, self.consumer_secret = self._validate_keys(
            consumer_keys, ["consumer_key", "consumer_secret"]
        ).values()
        self.token_id, self.token_secret = self._validate_keys(
            token_keys, ["token_key", "token_secret"]
        ).values()
        self._request_session = None

    def _validate_keys(self, keys: dict, key_names: list) -> dict:
        missing_keys = [k for k in key_names if k not in keys]
        if missing_keys:
            raise ValueError(f"Missing required keys: {missing_keys}")
        return {k: keys[k] for k in key_names}

    def _make_request_session(self) -> oauth.OAuth1Session:
        """
        Creates an OAuth1Session object for making requests to the NetSuite REST API.
        """
        return oauth.OAuth1Session(
            signature_method=self.signature_method,
            client_key=self.consumer_key,
            client_secret=self.consumer_secret,
            resource_owner_key=self.token_id,
            resource_owner_secret=self.token_secret,
            realm=self.account_id,
        )

    def _make_request(
        self,
        http_method: str,
        url: str,
        headers: Optional[dict[str, str]] = None,
        params: Optional[dict[str, Any]] = None,
        body: Optional[dict[str, Any]] = None,
    ) -> NetsuiteObject:
        """
        Makes an HTTP request to the NetSuite REST API using the specified HTTP method, URL, headers, parameters, and body.

        Parameters:
            http_method (str): The HTTP method to use for the request.
            url (str): The URL of the NetSuite REST API endpoint to which the request will be sent.
            headers (dict[str, str], optional): A dictionary of headers to include in the request. Defaults to {}.
            params (dict[str, Any], optional): A dictionary of parameters to include in the request. Defaults to {}.
            body (dict[str, Any], optional): A dictionary of data to include in the request body. Defaults to {}.

        Returns:
            NetsuiteObject: A NetsuiteObject containing the response data.
        """
        log.debug("Making request to restlet at %s.", url)
        log.debug("Payload: %s", body)
        log.debug("Headers: %s", json.dumps(headers))
        response = NetsuiteObject(url=url, request_headers=headers, request_data=body)
        try:
            self._request_session = self._make_request_session()

            method = getattr(self._request_session, http_method.lower())
            resp = method(
                url,
                data=(json.dumps(body) if isinstance(body, (dict, list)) else body),
                params=params,
                headers=headers,
            )
            log.debug("Got response headers: %s", json.dumps(dict(resp.headers)))
            response.response = resp.text
            response.code = resp.status_code
        except Exception:
            log.warning(traceback.format_exc())
            response.code = 500
            response.response = traceback.format_exc()
        return response

    def get(self, **kwargs) -> NetsuiteObject:
        """
        Makes a GET request to the NetSuite REST API using the specified URL, headers, and parameters.

        Parameters:
            http_method (str): The HTTP method to use for the request.
            url (str): The URL of the NetSuite REST API endpoint to which the request will be sent.
            headers (dict[str, str], optional): A dictionary of headers to include in the request. Defaults to {}.
            params (dict[str, Any], optional): A dictionary of parameters to include in the request. Defaults to {}.

        Returns:
            NetsuiteObject: A NetsuiteObject containing the response data.
        """
        return self._make_request(http_method="GET", **kwargs)

    def put(self, **kwargs) -> NetsuiteObject:
        """
        Makes a PUT request to the NetSuite REST API using the specified URL, headers, parameters, and body.

        Parameters:
            http_method (str): The HTTP method to use for the request.
            url (str): The URL of the NetSuite REST API endpoint to which the request will be sent.
            headers (dict[str, str], optional): A dictionary of headers to include in the request. Defaults to {}.
            params (dict[str, Any], optional): A dictionary of parameters to include in the request. Defaults to {}.
            body (dict[str, Any], optional): A dictionary of data to include in the request body. Defaults to {}.

        Returns:
            NetsuiteObject: A NetsuiteObject containing the response data.
        """
        return self._make_request(http_method="PUT", **kwargs)

    def post(self, **kwargs) -> NetsuiteObject:
        """
        Makes a POST request to the NetSuite REST API using the specified URL, headers, parameters, and body.

        Parameters:
            http_method (str): The HTTP method to use for the request.
            url (str): The URL of the NetSuite REST API endpoint to which the request will be sent.
            headers (dict[str, str], optional): A dictionary of headers to include in the request. Defaults to {}.
            params (dict[str, Any], optional): A dictionary of parameters to include in the request. Defaults to {}.
            body (dict[str, Any], optional): A dictionary of data to include in the request body. Defaults to {}.

        Returns:
            NetsuiteObject: A NetsuiteObject containing the response data.
        """
        return self._make_request(http_method="POST", **kwargs)

    def delete(self, **kwargs) -> NetsuiteObject:
        """
        Makes a DELETE request to the NetSuite REST API using the specified URL, headers, and parameters.

        Parameters:
            http_method (str): The HTTP method to use for the request.
            url (str): The URL of the NetSuite REST API endpoint to which the request will be sent.
            headers (dict[str, str], optional): A dictionary of headers to include in the request. Defaults to {}.
            params (dict[str, Any], optional): A dictionary of parameters to include in the request. Defaults to {}.
            body (dict[str, Any], optional): A dictionary of data to include in the request body. Defaults to {}.

        Returns:
            NetsuiteObject: A NetsuiteObject containing the response data.
        """
        return self._make_request(http_method="DELETE", **kwargs)



# Generated by CodiumAI
import json

import pytest
import requests

from NetSuite_Connector.NetSuite import NetSuite

"""
Code Analysis

Main functionalities:
The NetSuite class is a Python wrapper for the NetSuite REST API that provides methods for making HTTP requests to the NetSuite REST API using OAuth 1.0 authentication. It supports GET, PUT, POST, and DELETE HTTP methods and provides error handling for failed requests.

Methods:
- __init__(self, account_id: Any, consumer_keys: dict, token_keys: dict) -> None: Initializes the NetSuite object with the account ID, consumer keys, and token keys.
- get(self, **kwargs) -> NetsuiteObject: Makes a GET request to the NetSuite REST API using the specified URL, headers, and parameters. Returns a NetsuiteObject containing the response data.
- put(self, **kwargs) -> NetsuiteObject: Makes a PUT request to the NetSuite REST API using the specified URL, headers, parameters, and body. Returns a NetsuiteObject containing the response data.
- post(self, **kwargs) -> NetsuiteObject: Makes a POST request to the NetSuite REST API using the specified URL, headers, parameters, and body. Returns a NetsuiteObject containing the response data.
- delete(self, **kwargs) -> NetsuiteObject: Makes a DELETE request to the NetSuite REST API using the specified URL, headers, and parameters. Returns a NetsuiteObject containing the response data.

Fields:
- oauth_version: The OAuth version used for authentication.
- signature_method: The signature method used for authentication.
- account_id: The account ID used for authentication.
- consumer_key: The consumer key used for authentication.
- consumer_secret: The consumer secret used for authentication.
- token_key: The token ID used for authentication.
- token_secret: The token secret used for authentication.
- _request_session: The OAuth1Session object used for making requests to the NetSuite REST API.
"""


class TestNetSuite:
    # Tests that a GET request is successful.
    def test_successful_get_request(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        response_data = {"foo": "bar"}
        requests_mock.get(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.get(url=url, headers=headers, params=params)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that a PUT request is successful.
    def test_successful_put_request(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        body = {"foo": "bar"}
        response_data = {"success": True}
        requests_mock.put(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.put(url=url, headers=headers, params=params, body=body)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that an invalid URL raises an exception.
    def test_invalid_url(self):
        # Arrange
        url = "invalid_url"

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )

        # Assert
        with pytest.raises(Exception):
            ns.get(url=url)

    # Tests that an invalid HTTP method raises an exception.
    def test_invalid_http_method(self):
        # Arrange
        http_method = "INVALID_METHOD"

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )

        # Assert
        with pytest.raises(Exception):
            ns._make_request(
                http_method=http_method, url="", headers={}, params={}, body={}
            )

    # Tests that failed requests are handled properly.
    def test_failed_request_error_handling(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        body = {"foo": "bar"}
        requests_mock.get(url, exc=requests.exceptions.RequestException)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.get(url=url, headers=headers, params=params, body=body)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response is not None
        assert result.code == 500

    # Tests that a POST request is successful.
    def test_successful_post_request(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        body = {"foo": "bar"}
        response_data = {"success": True}
        requests_mock.post(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.post(url=url, headers=headers, params=params, body=body)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that a DELETE request is successful.
    def test_successful_delete_request(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        response_data = {"success": True}
        requests_mock.delete(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.delete(url=url, headers=headers, params=params)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that an empty headers dictionary does not raise an exception.
    def test_empty_headers(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {}
        params = {"param1": "value1"}
        response_data = {"success": True}
        requests_mock.get(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.get(url=url, headers=headers, params=params)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that an empty parameters dictionary does not raise an exception.
    def test_empty_parameters(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {}
        response_data = {"success": True}
        requests_mock.get(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.get(url=url, headers=headers, params=params)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that an empty body dictionary does not raise an exception.
    def test_empty_body(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        body = {}
        response_data = {"success": True}
        requests_mock.get(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.get(url=url, headers=headers, params=params, body=body)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that OAuth1 authentication is successful.
    def test_oauth1_authentication(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        response_data = {"success": True}
        requests_mock.get(url, json=response_data, status_code=200)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys=dict(consumer_key="2345678", consumer_secret="3456yhg"),
            token_keys=dict(token_key="wfdbfdsdfg", token_secret="efguhfjoidejhfije"),
        )
        result = ns.get(url=url, headers=headers, params=params)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 200

    # Tests that rate limiting is handled properly.
    def test_rate_limiting_handling(self, requests_mock):
        # Arrange
        url = "https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx"
        headers = {"Content-Type": "application/json"}
        params = {"param1": "value1"}
        response_data = {"success": True}
        requests_mock.get(url, json=response_data, status_code=429)

        # Act
        ns = NetSuite(
            account_id=123456,
            consumer_keys={"consumer_key": "", "consumer_secret": ""},
            token_keys={"token_key": "", "token_secret": ""},
        )
        result = ns.get(url=url, headers=headers, params=params)

        # Assert
        assert result.url == url
        assert result.request_headers == headers
        assert result.response == json.dumps(response_data)
        assert result.code == 429



import traceback
from typing import Any

from .NetSuite import NetSuite, NetsuiteObject


class ODBC(NetSuite):

    def __init__(self, account_id: Any, consumer_keys: dict, token_keys: dict) -> None:
        super().__init__(account_id, consumer_keys, token_keys)
        self.suiteql_endpoint = f'https://{account_id.lower().replace("_", "-")}.suitetalk.api.netsuite.com/services/rest/query/v1/suiteql'

    def query(self, query: str) -> NetsuiteObject:
        """
        Perfom a query to ODBC driver
        query: fully qualified sql query
        >>> from NetSuite_Connector.ODBC import ODBC

        >>> nt = ODBC(
            account_id=123456,
            consumer_keys=dict(consumer_key="2345678", consumer_secret="3456yhg"),
            token_keys=dict(token_key="wfdbfdsdfg", token_secret="efguhfjoidejhfije")
            )

        >>> q = nt.query("SELECT top 10 * FROM transaction")

        >>> NetsuiteObject(url='https://xxxx.restlets.api.netsuite.com/app/site/hosting/restlet.nl?script=xxxx&deploy=xxxx', request_headers={'Content-Type': 'application/json'}, response='{"foo":"bar"}', code=200)
        """
        response = NetsuiteObject(request_data=query)
        try:
            data = {"q": query}
            headers = {"prefer": "transient", "Content-Type": "application/json"}
            req = self.post(url=self.suiteql_endpoint, body=data, headers=headers)
            response.response = req.response
            response.code = req.code
        except Exception:
            response.code = 500
            response.response = traceback.format_exc()

        return response



# Generated by CodiumAI
import pytest


from src.NetSuite_Connector.ODBC import ODBC


class TestODBC:

    # ODBC can be instantiated with valid account_id, consumer_keys, and token_keys.
    def test_instantiation_with_valid_keys(self):
        account_id = "123456"
        consumer_keys = {"consumer_key": "2345678", "consumer_secret": "3456yhg"}
        token_keys = {"token_key": "wfdbfdsdfg", "token_secret": "efguhfjoidejhfije"}

        odbc = ODBC(account_id, consumer_keys, token_keys)

        assert odbc.account_id == account_id
        assert odbc.consumer_key == consumer_keys["consumer_key"]
        assert odbc.consumer_secret == consumer_keys["consumer_secret"]
        assert odbc.token_id == token_keys["token_key"]
        assert odbc.token_secret == token_keys["token_secret"]

    # ODBC raises a ValueError if instantiated with missing required keys.
    def test_instantiation_with_missing_keys(self):
        account_id = 123456
        consumer_keys = {"consumer_key": "2345678"}
        token_keys = {"token_key": "wfdbfdsdfg", "token_secret": "efguhfjoidejhfije"}

        with pytest.raises(ValueError):
            ODBC(account_id, consumer_keys, token_keys)
